@@ -101,7 +101,9 @@ static struct info_cache_t info_cache;
 static char * skipMultipleSlashes(char * requ, size_t* len) {
   char * r = requ;
 
-  if ((r[0] == '\0') // zero length
+  if (!r // null pointer
+      || (len && !*len) // zero length
+      || (r[0] == '\0') // zero length
       || (r[0] != '/') // does not start with a slash
       || (r[1] != '/')) // does not have another slash
   {
@@ -142,11 +144,11 @@ static unsigned long long SIW_ENTRIES_ALL[] = {
     //
     SIW_OLSRD_CONF, //
     //
-    SIW_NETJSON_NETWORK_ROUTES,
-    SIW_NETJSON_NETWORK_GRAPH,
-    SIW_NETJSON_DEVICE_CONFIGURATION,
-    SIW_NETJSON_DEVICE_MONITORING,
-    SIW_NETJSON_NETWORK_COLLECTION
+    SIW_NETJSON_NETWORK_ROUTES, //
+    SIW_NETJSON_NETWORK_GRAPH, //
+    SIW_NETJSON_DEVICE_CONFIGURATION, //
+    SIW_NETJSON_DEVICE_MONITORING, //
+    SIW_NETJSON_NETWORK_COLLECTION //
     };
 
 long cache_timeout_generic(info_plugin_config_t *plugin_config, unsigned long long siw) {
@@ -306,16 +308,16 @@ static unsigned int determine_action(char *requ) {
   }
 }
 
-static void send_status_no_retries(const char * req, int the_socket, unsigned int status) {
+static void send_status_no_retries(const char * req, bool add_headers, int the_socket, unsigned int status) {
   struct autobuf abuf;
 
   abuf_init(&abuf, AUTOBUFCHUNK);
 
-  if (config->http_headers) {
+  if (add_headers) {
     http_header_build_result(status, &abuf);
   } else if (status != INFO_HTTP_OK) {
     if (functions->output_error) {
-      functions->output_error(&abuf, status, req, config->http_headers);
+      functions->output_error(&abuf, status, req, add_headers);
     } else if (status == INFO_HTTP_NOCONTENT) {
       /* wget can't handle output of zero length */
       abuf_puts(&abuf, "\n");
@@ -459,7 +461,7 @@ static void send_info_from_table(struct autobuf *abuf, unsigned int send_what, S
   }
 }
 
-static void send_info(const char * req, unsigned int send_what, int the_socket, unsigned int status) {
+static void send_info(const char * req, bool add_headers, unsigned int send_what, int the_socket, unsigned int status) {
   struct autobuf abuf;
   unsigned int outputLength = 0;
   unsigned int send_index = 0;
@@ -473,7 +475,7 @@ static void send_info(const char * req, unsigned int send_what, int the_socket,
 
   abuf_init(&abuf, AUTOBUFCHUNK);
 
-  if (config->http_headers) {
+  if (add_headers) {
     http_header_build(name, status, content_type, &abuf, &contentLengthIndex);
     headerLength = abuf.len;
   }
@@ -523,7 +525,7 @@ static void send_info(const char * req, unsigned int send_what, int the_socket,
       status = INFO_HTTP_NOCONTENT;
       abuf.buf[0] = '\0';
       abuf.len = 0;
-      if (config->http_headers) {
+      if (add_headers) {
         http_header_build(name, status, content_type, &abuf, &contentLengthIndex);
         headerLength = abuf.len;
       }
@@ -532,14 +534,14 @@ static void send_info(const char * req, unsigned int send_what, int the_socket,
 
   if (status != INFO_HTTP_OK) {
     if (functions->output_error) {
-      functions->output_error(&abuf, status, req, config->http_headers);
+      functions->output_error(&abuf, status, req, add_headers);
     } else if (status == INFO_HTTP_NOCONTENT) {
       /* wget can't handle output of zero length */
       abuf_puts(&abuf, "\n");
     }
   }
 
-  if (config->http_headers) {
+  if (add_headers) {
     http_header_adjust_content_length(&abuf, contentLengthIndex, abuf.len - headerLength);
   }
 
@@ -580,6 +582,10 @@ static void send_info(const char * req, unsigned int send_what, int the_socket,
 }
 
 static char * skipLeadingWhitespace(char * requ, size_t *len) {
+  if (!requ || !len || !*len) {
+    return requ;
+  }
+
   while (isspace(*requ) && (*requ != '\0')) {
     *len = *len - 1;
     requ++;
@@ -587,7 +593,11 @@ static char * skipLeadingWhitespace(char * requ, size_t *len) {
   return requ;
 }
 
-static char * stripEOLs(char * requ, size_t *len) {
+static char * stripTrailingWhitespace(char * requ, size_t *len) {
+  if (!requ || !len || !*len) {
+    return requ;
+  }
+
   while (isspace(requ[*len - 1]) && (requ[*len - 1] != '\0')) {
     *len = *len - 1;
     requ[*len] = '\0';
@@ -596,6 +606,10 @@ static char * stripEOLs(char * requ, size_t *len) {
 }
 
 static char * stripTrailingSlashes(char * requ, size_t *len) {
+  if (!requ || !len || !*len) {
+    return requ;
+  }
+
   while ((requ[*len - 1] == '/') && (requ[*len - 1] != '\0')) {
     *len = *len - 1;
     requ[*len] = '\0';
@@ -606,6 +620,11 @@ static char * stripTrailingSlashes(char * requ, size_t *len) {
 static char * cutAtFirstEOL(char * requ, size_t *len) {
   char * s = requ;
   size_t l = 0;
+
+  if (!requ || !len || !*len) {
+    return requ;
+  }
+
   while (!((*s == '\n') || (*s == '\r')) && (*s != '\0')) {
     s++;
     l++;
@@ -617,50 +636,79 @@ static char * cutAtFirstEOL(char * requ, size_t *len) {
   return requ;
 }
 
-static char * parseRequest(char * requ, size_t *len) {
-  char * req = requ;
-
-  if (!req || !*len) {
-    return requ;
-  }
-
-  req = skipLeadingWhitespace(req, len);
-  req = stripEOLs(req, len);
-
-  /* HTTP request: GET whitespace URI whitespace HTTP/1.1 */
-  if (*len < (3 + 1 + 1 + 1 + 8)) {
+static char * parseRequest(char * req, size_t *len) {
+  if (!req || !len || !*len) {
     return req;
   }
 
-  if (strncasecmp(req, "GET", 3) || !isspace(req[3])) {
-    /* does not start with 'GET ' */
-    return req;
-  }
-
-  /* skip 'GET ' and further leading whitespace */
-  req = skipLeadingWhitespace(&req[4], len);
-  if (!*len) return req;
-
-  /* cut req at the first '\n' */
-  req = cutAtFirstEOL(req, len);
-  if (!*len) return req;
-
-  /* strip req of trailing EOL and whitespace */
-  req = stripEOLs(req, len);
-  if (*len < 9) return req;
-
-  if (!isspace(req[*len - 9]) //
+  /* HTTP request: GET whitespace URI whitespace HTTP/1.[01] */
+  if (*len < (3 + 1 + 1 + 1 + 8) //
+      || strncasecmp(req, "GET", 3) //
+      || !isspace(req[3]) //
+      || !isspace(req[*len - 9]) //
       || strncasecmp(&req[*len - 8], "HTTP/1.", 7) //
       || ((req[*len - 1] != '1') && (req[*len - 1] != '0'))) {
+    /* too short or does not start with 'GET ' nor ends with ' HTTP/1.[01]'*/
     return req;
   }
-  *len = *len - 8;
-  req[*len] = '\0';
-  if (!*len) return req;
 
-  /* strip req of trailing EOL and whitespace */
-  req = stripEOLs(req, len);
-  req = stripTrailingSlashes(req, len);
+  /* skip 'GET ' */
+  *len = *len - 4;
+  req = &req[4];
+
+  /* strip ' HTTP/1.[01]' */
+  *len = *len - 9;
+  req[*len] = '\0';
+
+  return req;
+}
+
+static char * checkCommandPrefixes(char * req, size_t *len, bool *add_headers) {
+  size_t l;
+
+  if (!req || !len || !*len || !add_headers) {
+    return req;
+  }
+
+  l = *len;
+
+  /* '/http/...' */
+
+  if ((l >= (SIW_PREFIX_HTTP_LEN + 1)) && !strncasecmp(req, SIW_PREFIX_HTTP "/", SIW_PREFIX_HTTP_LEN + 1)) {
+    *len = l - SIW_PREFIX_HTTP_LEN;
+    req = &req[SIW_PREFIX_HTTP_LEN];
+    *add_headers = true;
+    return req;
+  }
+
+  /* '/http' */
+
+  if ((l == SIW_PREFIX_HTTP_LEN) && !strcasecmp(req, SIW_PREFIX_HTTP)) {
+    *len = 0;
+    *req = '\0';
+    *add_headers = true;
+    return req;
+  }
+
+  /* '/plain/...' */
+
+  if ((l >= (SIW_PREFIX_PLAIN_LEN + 1)) && !strncasecmp(req, SIW_PREFIX_PLAIN "/", SIW_PREFIX_PLAIN_LEN + 1)) {
+    *len = l - SIW_PREFIX_PLAIN_LEN;
+    req = &req[SIW_PREFIX_PLAIN_LEN];
+    *add_headers = false;
+    return req;
+  }
+
+  /* '/plain' */
+
+  if ((l == SIW_PREFIX_PLAIN_LEN) && !strcasecmp(req, SIW_PREFIX_PLAIN)) {
+    *len = 0;
+    *req = '\0';
+    *add_headers = false;
+    return req;
+  }
+
+  /* no prefixes */
 
   return req;
 }
@@ -668,11 +716,17 @@ static char * parseRequest(char * requ, size_t *len) {
 static void drain_request(int ipc_connection) {
   static char drain_buffer[AUTOBUFCHUNK];
 
-  /* input was much too long: read until the end for graceful connection termination
-   * because wget can't handle the premature connection termination that is allowed
-   * by the INFO_HTTP_REQUEST_ENTITY_TOO_LARGE HTTP status code
-   */
-  while (recv(ipc_connection, (void *) &drain_buffer, sizeof(drain_buffer), 0) == sizeof(drain_buffer)) {}
+  ssize_t r;
+  do {
+#ifdef _WIN32
+    r = recv(ipc_connection, (void *) &drain_buffer, sizeof(drain_buffer), MSG_PEEK);
+    if (r > 0) {
+      r = recv(ipc_connection, (void *) &drain_buffer, sizeof(drain_buffer), 0);
+    }
+#else
+    r = recv(ipc_connection, (void *) &drain_buffer, sizeof(drain_buffer), MSG_DONTWAIT);
+#endif
+  } while ((r > 0) && (r <= (ssize_t) sizeof(drain_buffer)));
 }
 
 static void ipc_action(int fd, void *data __attribute__ ((unused)), unsigned int flags __attribute__ ((unused))) {
@@ -686,11 +740,15 @@ static void ipc_action(int fd, void *data __attribute__ ((unused)), unsigned int
   bool hostDenied = false;
   struct timeval timeout;
   fd_set read_fds;
-  char req_buffer[1024];
+  char req_buffer[1024]; /* maximum size is the size of an IP packet */
   char * req = req_buffer;
   ssize_t rx_count = 0;
   unsigned int send_what = 0;
   unsigned int http_status = INFO_HTTP_OK;
+  bool add_headers = config->http_headers;
+  int r = 0;
+
+  *req = '\0';
 
   if ((ipc_connection = accept(fd, &sock_addr.in, &sock_addr_len)) < 0) {
 #ifndef NODEBUG
@@ -700,9 +758,102 @@ static void ipc_action(int fd, void *data __attribute__ ((unused)), unsigned int
     return;
   }
 
+  /* Wait at most this much time for the request to arrive on the connection */
+  timeout.tv_sec = (outbuffer.count >= MAX_CLIENTS) ? 0 : config->request_timeout_sec;
+  timeout.tv_usec = (outbuffer.count >= MAX_CLIENTS) ? 0 : config->request_timeout_usec;
+
+  FD_ZERO(&read_fds);
+#ifndef _WIN32
+  FD_SET(ipc_connection, &read_fds);
+#else
+  FD_SET((unsigned int ) ipc_connection, &read_fds); /* Win32 needs the cast here */
+#endif
+
+  /* On success, select() and pselect() return the number of file descriptors
+   * contained in the three returned descriptor sets (that is, the total number
+   * of bits that are set in readfds, writefds, exceptfds) which may be zero if
+   * the timeout expires before anything interesting happens. On error, -1 is
+   * returned, and errno is set to indicate the error; the file descriptor sets
+   * are unmodified, and timeout becomes undefined.
+   */
+
+  r = select(ipc_connection + 1, &read_fds, NULL, NULL, &timeout);
+  if (r < 0) {
+    /* ipc_connection is not ready for reading */
+#ifndef NODEBUG
+    olsr_printf(1, "(%s) select()=%s\n", name, strerror(errno));
+#endif /* NODEBUG */
+    drain_request(ipc_connection);
+    if (outbuffer.count >= MAX_CLIENTS) {
+      send_status_no_retries(req, add_headers, ipc_connection, INFO_HTTP_INTERNAL_SERVER_ERROR);
+    } else {
+      send_info(req, add_headers, send_what, ipc_connection, INFO_HTTP_INTERNAL_SERVER_ERROR);
+    }
+    return;
+  }
+
+  if (!r) {
+    /* ipc_connection is not ready for reading within the timeout */
+#ifndef NODEBUG
+    olsr_printf(1, "(%s) select() timeout\n", name);
+#endif /* NODEBUG */
+    drain_request(ipc_connection);
+    if (outbuffer.count >= MAX_CLIENTS) {
+      send_status_no_retries(req, add_headers, ipc_connection, INFO_HTTP_REQUEST_TIMEOUT);
+    } else {
+      send_info(req, add_headers, send_what, ipc_connection, INFO_HTTP_REQUEST_TIMEOUT);
+    }
+    return;
+  }
+
+#ifdef _WIN32
+  rx_count = recv(ipc_connection, req, sizeof(req_buffer), MSG_PEEK);
+  if (rx_count > 0) {
+    rx_count = recv(ipc_connection, req, sizeof(req_buffer), 0);
+  }
+#else
+  rx_count = recv(ipc_connection, req, sizeof(req_buffer), MSG_DONTWAIT);
+#endif
+
+  /* Upon successful completion, recv() shall return the length of the message
+   * in bytes. If no messages are available to be received and the peer has
+   * performed an orderly shutdown, recv() shall return 0. Otherwise, −1 shall
+   * be returned and errno set to indicate the error.
+   */
+
+  /* ensure proper request termination */
+  if (rx_count <= 0) {
+    *req = '\0';
+  } else if (rx_count < (ssize_t) sizeof(req_buffer)) {
+    req[rx_count] = '\0';
+  } else {
+    req[sizeof(req_buffer) - 1] = '\0';
+  }
+
+  /* sanitise the request */
+  if (rx_count > 0) {
+    req = cutAtFirstEOL(req, (size_t*) &rx_count);
+
+    req = stripTrailingWhitespace(req, (size_t*) &rx_count);
+    req = skipLeadingWhitespace(req, (size_t*) &rx_count);
+
+    /* detect http requests */
+    req = parseRequest(req, (size_t*) &rx_count);
+
+    req = stripTrailingWhitespace(req, (size_t*) &rx_count);
+    req = stripTrailingSlashes(req, (size_t*) &rx_count);
+    req = skipLeadingWhitespace(req, (size_t*) &rx_count);
+    req = skipMultipleSlashes(req, (size_t*) &rx_count);
+
+    req = checkCommandPrefixes(req, (size_t*) &rx_count, &add_headers);
+
+    req = skipMultipleSlashes(req, (size_t*) &rx_count);
+  }
+
   if (outbuffer.count >= MAX_CLIENTS) {
     /* limit the number of replies that are in-flight */
-    send_status_no_retries(req, ipc_connection, INFO_HTTP_SERVICE_UNAVAILABLE);
+    drain_request(ipc_connection);
+    send_status_no_retries(req, add_headers, ipc_connection, INFO_HTTP_SERVICE_UNAVAILABLE);
     return;
   }
 
@@ -735,56 +886,20 @@ static void ipc_action(int fd, void *data __attribute__ ((unused)), unsigned int
     olsr_printf(1, "(%s) Connect from host %s is not allowed!\n", name, addr);
 #endif /* NODEBUG */
     drain_request(ipc_connection);
-    send_info(req, send_what, ipc_connection, INFO_HTTP_FORBIDDEN);
+    send_info(req, add_headers, send_what, ipc_connection, INFO_HTTP_FORBIDDEN);
     return;
   }
 
 #ifndef NODEBUG
-  olsr_printf(2, "(%s) Connect from host %s is allowed\n", name, addr);
+  olsr_printf(1, "(%s) Connect from host %s is allowed\n", name, addr);
 #endif /* NODEBUG */
 
-  timeout.tv_sec = timeout.tv_usec = 0;
-
-  FD_ZERO(&read_fds);
-#ifndef _WIN32
-  FD_SET(ipc_connection, &read_fds);
-#else
-  FD_SET((unsigned int ) ipc_connection, &read_fds);
-#endif
-
-  /* On success, select() and pselect() return the number of file descriptors
-   * contained in the three returned descriptor sets (that is, the total number
-   * of bits that are set in readfds, writefds, exceptfds) which may be zero if
-   * the timeout expires before anything interesting happens. On error, -1 is
-   * returned, and errno is set to indicate the error; the file descriptor sets
-   * are unmodified, and timeout becomes undefined.
-   */
-
-  if (select(ipc_connection + 1, &read_fds, NULL, NULL, &timeout) < 0) {
-#ifndef NODEBUG
-    olsr_printf(1, "(%s) select()=%s\n", name, strerror(errno));
-#endif /* NODEBUG */
-    drain_request(ipc_connection);
-    send_info(req, send_what, ipc_connection, INFO_HTTP_INTERNAL_SERVER_ERROR);
-    return;
-  }
-
-  *req = '\0';
-  rx_count = recv(ipc_connection, req, sizeof(req_buffer), 0); /* Win32 needs the cast here */
-
-  /* Upon successful completion, recv() shall return the length of the message
-   * in bytes. If no messages are available to be received and the peer has
-   * performed an orderly shutdown, recv() shall return 0. Otherwise, −1 shall
-   * be returned and errno set to indicate the error.
-   */
-
   if (rx_count < 0) {
 #ifndef NODEBUG
-    olsr_printf(1, "(%s) s < 0\n", name);
+    olsr_printf(1, "(%s) rx_count < 0\n", name);
 #endif /* NODEBUG */
-    *req = '\0';
     drain_request(ipc_connection);
-    send_info(req, send_what, ipc_connection, INFO_HTTP_INTERNAL_SERVER_ERROR);
+    send_info(req, add_headers, send_what, ipc_connection, INFO_HTTP_INTERNAL_SERVER_ERROR);
     return;
   }
 
@@ -792,11 +907,10 @@ static void ipc_action(int fd, void *data __attribute__ ((unused)), unsigned int
 
   if (!rx_count) {
 #ifndef NODEBUG
-    olsr_printf(1, "(%s) s == 0\n", name);
+    olsr_printf(1, "(%s) rx_count == 0\n", name);
 #endif /* NODEBUG */
-    *req = '\0';
     drain_request(ipc_connection);
-    send_info(req, send_what, ipc_connection, INFO_HTTP_NOCONTENT);
+    send_info(req, add_headers, SIW_EVERYTHING, ipc_connection, INFO_HTTP_OK);
     return;
   }
 
@@ -804,21 +918,22 @@ static void ipc_action(int fd, void *data __attribute__ ((unused)), unsigned int
 
   if (rx_count >= (ssize_t) sizeof(req_buffer)) {
 #ifndef NODEBUG
-    olsr_printf(1, "(%s) s > %ld\n", name, (long int) sizeof(req_buffer));
+    olsr_printf(1, "(%s) rx_count > %ld\n", name, (long int) sizeof(req_buffer));
 #endif /* NODEBUG */
-    req[sizeof(req_buffer) - 1] = '\0';
+
+    /* input was much too long: read until the end for graceful connection termination
+     * because wget can't handle the premature connection termination that is allowed
+     * by the INFO_HTTP_REQUEST_ENTITY_TOO_LARGE HTTP status code
+     */
     drain_request(ipc_connection);
-    send_info(req, send_what, ipc_connection, INFO_HTTP_REQUEST_ENTITY_TOO_LARGE);
+    send_info(req, add_headers, send_what, ipc_connection, INFO_HTTP_REQUEST_ENTITY_TOO_LARGE);
     return;
   }
 
   /* 0 < rx_count < sizeof(requ) */
 
-  req[rx_count] = '\0';
-  req = parseRequest(req, (size_t*) &rx_count);
-  req = skipMultipleSlashes(req, (size_t*) &rx_count);
-  if ((req[0] == '\0') //
-      || ((req[0] == '/') && (req[1] == '\0'))) {
+  if (!rx_count //
+      || ((rx_count == 1) && (*req == '/'))) {
     /* empty or '/' */
     send_what = SIW_EVERYTHING;
   } else {
@@ -829,7 +944,7 @@ static void ipc_action(int fd, void *data __attribute__ ((unused)), unsigned int
     http_status = INFO_HTTP_NOTFOUND;
   }
 
-  send_info(req, send_what, ipc_connection, http_status);
+  send_info(req, add_headers, send_what, ipc_connection, http_status);
 }
 
 static int plugin_ipc_init(void) {
@@ -907,7 +1022,7 @@ static int plugin_ipc_init(void) {
   add_olsr_socket(ipc_socket, &ipc_action, NULL, NULL, SP_PR_READ);
 
 #ifndef NODEBUG
-  olsr_printf(2, "(%s) listening on port %d\n", name, config->ipc_port);
+  olsr_printf(1, "(%s) listening on port %d\n", name, config->ipc_port);
 #endif /* NODEBUG */
 
   return 1;
@@ -920,6 +1035,19 @@ static int plugin_ipc_init(void) {
   return 0;
 }
 
+static void info_sanitise_config(info_plugin_config_t *cfg) {
+  if (cfg->ipc_port < 1) {
+    cfg->ipc_port = 1;
+  }
+
+  if (cfg->request_timeout < 0) {
+    cfg->request_timeout = 0;
+  }
+
+  cfg->request_timeout_sec = cfg->request_timeout / 1000;
+  cfg->request_timeout_usec = (cfg->request_timeout % 1000) * 1000;
+}
+
 int info_plugin_init(const char * plugin_name, info_plugin_functions_t *plugin_functions, info_plugin_config_t *plugin_config) {
   int i;
 
@@ -931,6 +1059,8 @@ int info_plugin_init(const char * plugin_name, info_plugin_functions_t *plugin_f
   functions = plugin_functions;
   config = plugin_config;
 
+  info_sanitise_config(config);
+
   memset(&outbuffer, 0, sizeof(outbuffer));
   for (i = 0; i < MAX_CLIENTS; ++i) {
     outbuffer.socket[i] = -1;
