@@ -1,14 +1,16 @@
-
 /*
- * Secure OLSR plugin
- * http://www.olsr.org
+ * The olsr.org Optimized Link-State Routing daemon (olsrd)
+ *
+ * (c) by the OLSR project
+ *
+ * See our Git repository to find out who worked on this file
+ * and thus is a copyright holder on it.
  *
- * Copyright (c) 2004, Andreas Tonnesen(andreto@olsr.org)
  * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or
- * without modification, are permitted provided that the following
- * conditions are met:
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
  * * Redistributions of source code must retain the above copyright
  *   notice, this list of conditions and the following disclaimer.
@@ -16,7 +18,7 @@
  *   notice, this list of conditions and the following disclaimer in
  *   the documentation and/or other materials provided with the
  *   distribution.
- * * Neither the name of olsrd, olsr.org nor the names of its
+ * * Neither the name of olsr.org, olsrd nor the names of its
  *   contributors may be used to endorse or promote products derived
  *   from this software without specific prior written permission.
  *
@@ -33,6 +35,12 @@
  * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  *
+ * Visit http://www.olsr.org for more information.
+ *
+ * If you find this software useful feel free to make a donation
+ * to the project. For more information see the website or contact
+ * the copyright holders.
+ *
  */
 
 /* Adjustments made to ensure data going out is converted to network
@@ -179,12 +187,14 @@ secure_plugin_init(void)
   i = read_key_from_file(keyfile);
 
   if (i < 0) {
-    olsr_printf(1, "[ENC]Could not read key from file %s!\nExitting!\n\n", keyfile);
-    exit(1);
+    char buf[1024];
+    snprintf(buf, sizeof(buf), "SECURE: Could not read key from file %s", keyfile);
+    olsr_exit(buf, EXIT_FAILURE);
   }
   if (i == 0) {
-    olsr_printf(1, "[ENC]There was a problem reading key from file %s. Is the key long enough?\nExitting!\n\n", keyfile);
-    exit(1);
+    char buf[1024];
+    snprintf(buf, sizeof(buf), "SECURE: There was a problem reading key from file %s. Is the key long enough?", keyfile);
+    olsr_exit(buf, EXIT_FAILURE);
   }
 
   /* Register the packet transform function */
@@ -536,7 +546,7 @@ send_challenge(struct interface_olsr *olsr_if, const union olsr_ip_addr *new_hos
 
   /* Set the size including OLSR packet size */
 
-  challenge = olsr_random() << 16;
+  challenge = (uint32_t)olsr_random() << 16;
   challenge |= olsr_random();
 
   /* initialise rrmsg */
