@@ -114,10 +114,10 @@ public class Builder extends Analyzer {
 					manifest = new Manifest(in);
 					in.close();
 				} catch (Exception e) {
-					error(MANIFEST + " while reading manifest file", e);
+					exception(e, "%s: exception while reading manifest file", MANIFEST);
 				}
 			} else {
-				error(MANIFEST + ", no such file %s", mf);
+				error("%s: no such file %s", MANIFEST, mf);
 			}
 		}
 
@@ -489,7 +489,7 @@ public class Builder extends Analyzer {
 						}
 					}
 					if (packageRef.isDefaultPackage())
-						System.err.println("Duh?");
+						trace("Package reference is default package");
 					dot.putResource("OSGI-OPT/src/" + sourcePath, new FileResource(f));
 				}
 			}
@@ -1295,7 +1295,7 @@ public class Builder extends Analyzer {
 				result.add(jar);
 				doneBuild(builder);
 			} catch (Exception e) {
-				builder.error("Sub Building %s", e, builder.getBsn());
+				builder.exception(e, "Exception Building %s", builder.getBsn());
 			}
 			if (builder != this)
 				getInfo(builder, builder.getBsn() + ": ");
@@ -1391,8 +1391,7 @@ public class Builder extends Analyzer {
 	public Builder getSubBuilder() throws Exception {
 		Builder builder = new Builder(this);
 		builder.setBase(getBase());
-		builder.setPedantic(isPedantic());
-		builder.setTrace(isTrace());
+		builder.use(this);
 
 		for (Jar file : getClasspath()) {
 			builder.addClasspath(file);
