@@ -162,6 +162,10 @@ olsrmain_load_config(char *file) {
     fprintf(stderr, "Error while reading config file %s!\n", file);
     return -1;
   }
+
+  free(olsr_cnf->configuration_file);
+  olsr_cnf->configuration_file = strdup(file);
+
   return 0;
 }
 
@@ -1070,6 +1074,7 @@ olsrd_get_default_cnf(char * configuration_file)
     return NULL;
   }
 
+  memset(c, 0, sizeof(struct olsrd_config));
   set_default_cnf(c, configuration_file);
   return c;
 }
