@@ -49,14 +49,9 @@
 #include <sys/stat.h>
 #include <assert.h>
 #include <fcntl.h>
-#if defined(__linux__) && !defined(__ANDROID__)
-  #ifdef __GLIBC__
-    #include <execinfo.h>
-  #endif
-#include <syslog.h>
-#endif /* defined(__linux__) && !defined(__ANDROID__) */
 
 #include "cfgparser/olsrd_conf.h"
+#include "cfgparser/olsrd_conf_checksum.h"
 #include "ipcalc.h"
 #include "defs.h"
 #include "builddata.h"
@@ -79,6 +74,14 @@
 #include "lock_file.h"
 #include "cli.h"
 
+#if defined(__GLIBC__) && defined(__linux__) && !defined(__ANDROID__) && !defined(__UCLIBC__)
+  #define OLSR_HAVE_EXECINFO_H
+#endif
+
+#ifdef OLSR_HAVE_EXECINFO_H
+  #include <execinfo.h>
+#endif /* OLSR_HAVE_EXECINFO_H */
+
 #ifdef __linux__
 #include <linux/types.h>
 #include <linux/rtnetlink.h>
@@ -102,7 +105,7 @@ static char **olsr_argv = NULL;
 struct olsr_cookie_info *def_timer_ci = NULL;
 
 static void printStacktrace(const char * message) {
-#ifdef __GLIBC__
+#ifdef OLSR_HAVE_EXECINFO_H
   void *bt_array[64];
   size_t bt_size;
   size_t bt_index=0;
@@ -117,7 +120,7 @@ static void printStacktrace(const char * message) {
   }
 #else
   olsr_syslog(OLSR_LOG_ERR, "%s (logging a stack trace is not supported on this platform)", message);
-#endif
+#endif /* OLSR_HAVE_EXECINFO_H */
 }
 
 #ifndef _WIN32
@@ -401,12 +404,39 @@ static int set_default_ifcnfs(struct olsr_if *ifs, struct if_config_options *cnf
  * Main entrypoint
  */
 
+static int argc_saved = 0;
+char ** argv_saved = NULL;
+
+void get_argc_argv(int *argc, char ***argv) {
+  if (argc) {
+    *argc = argc_saved;
+  }
+  if (argv) {
+    *argv = argv_saved;
+  }
+}
+
 int main(int argc, char *argv[]) {
   int argcLocal = argc;
 
+  /* save argc and argv */
+  {
+    size_t i;
+
+    argc_saved = argc;
+    argv_saved = malloc(sizeof(char*) * (argc + 1));
+    for (i = 0; i <= (size_t) argc; i++) {
+      argv_saved[i] = argv[i];
+    }
+  }
+
   /* Open syslog */
   olsr_openlog("olsrd");
 
+  olsrd_config_checksum_init();
+  olsrd_config_checksum_add_cli(argc, argv);
+  olsrd_config_checksum_add(OLSRD_CONFIG_START, OLSRD_CONFIG_START_LEN);
+
   /*
    * Initialisation
    */
@@ -493,6 +523,9 @@ int main(int argc, char *argv[]) {
     free(default_ifcnf);
   }
 
+  olsrd_config_checksum_add(OLSRD_CONFIG_END, OLSRD_CONFIG_END_LEN);
+  olsrd_config_checksum_final();
+
   /* Sanity check configuration */
   if (olsrd_sanity_check_cnf(olsr_cnf) < 0) {
     char buf2[1024];
