@@ -59,6 +59,10 @@
 #include <assert.h>
 #include <time.h>
 
+#ifdef __MACH__
+#include "mach/clock_gettime.h"
+#endif
+
 #ifdef _WIN32
 #define close(x) closesocket(x)
 #endif /* _WIN32 */
@@ -610,7 +614,7 @@ calc_jitter(unsigned int rel_time, uint8_t jitter_pct, unsigned int random_val)
    * Play some tricks to avoid overflows with integer arithmetic.
    */
   jitter_time = (jitter_pct * rel_time) / 100;
-  jitter_time = random_val / (1 + RAND_MAX / (jitter_time + 1));
+  jitter_time = random_val / (1 + OLSR_RANDOM_MAX / (jitter_time + 1));
 
   OLSR_PRINTF(3, "TIMER: jitter %u%% rel_time %ums to %ums\n", jitter_pct, rel_time, rel_time - jitter_time);
 
